class Player:
    def __init__(self,
                 name: str,
                 clan: str,
                 country: int,
                 score: int,
                 is_spectator: bool):
        self.name = name
        self.clan = clan
        self.country = country
        self.score = score
        self.is_spectator = is_spectator

    def __str__(self):
        return f"{self.name} [{self.clan}]"



from typing import List
from .player import Player


class ServerInfo:
    def __init__(self,
                 version: str=None,
                 name: str=None,
                 _map: str=None,
                 game_type: str=None,
                 password: bool=None,
                 player_count: int=None,
                 max_player_count: int=None,
                 client_count: int=None,
                 max_client_count: int=None,
                 mapcrc: int=None,
                 mapsize: int=None,
                 players: List[Player]=None):
        self.name = name
        self.map = _map
        self.version = version
        self.game_type = game_type
        self.password = password
        self.player_count = player_count
        self.max_player_count = max_player_count
        self.client_count = client_count
        self.max_client_count = max_client_count
        self.mapcrc = mapcrc
        self.mapsize = mapsize
        self.players = players

        if self.players is None:
            self.players: List[Player] = []

    def __str__(self):
        return f"{self.name} [{self.player_count}/{self.max_player_count}]"



import socket
import os
import sys
from .serverinfo import ServerInfo
from .player import Player


def create_info_packet():
    data = bytearray()
    data.extend(map(ord, "xe"))
    extra_token = os.urandom(2)
    token = os.urandom(1)
    data += extra_token
    for x in range(6):
        data.append(0xff)
    data.extend(map(ord, "gie3"))
    data += token
    return data, token, extra_token


class ServerHandler:
    def __init__(self,
                 address: str,
                 port: int=8303,
                 ignore_token: bool=False,
                 timeout: int=5):
        self.address = address
        self.port = port
        self.ignore_token = ignore_token
        self.timeout = timeout

    async def get_info(self):
        sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        sock.settimeout(self.timeout)
        data, token, extra_token = create_info_packet()

        extra_token = int.from_bytes(extra_token, byteorder=sys.byteorder)
        token = int.from_bytes(token, byteorder=sys.byteorder)

        sock.sendto(data, (self.address, self.port))

        data, (address, port) = sock.recvfrom(1024)
        sock.close()
        if address != self.address:
            return

        svtype = data[10:14].decode()
        stype = ""

        if svtype == "inf3":
            svtype = "vanilla"
        elif svtype == "dtsf":
            svtype = "64legacy"
        elif svtype == "iext":
            svtype = "ext"
        elif svtype == "iex+":
            svtype = "extmore"
            stype = "ext"

        data = data[14:]
        slots: list = data.split("\x00".encode())

        received_token = int(slots.pop(0))

        if not self.ignore_token:
            if token != (received_token & 0xff):
                raise AssertionError("Token is invalid.")

            if stype == "ext" and extra_token != (token & 0xffff00) >> 8:
                raise AssertionError("Ext Token is invalid.")

        info = ServerInfo()

        if svtype != "extmore":
            info.version = slots.pop(0).decode()
            info.name = slots.pop(0).decode()
            info.map = slots.pop(0).decode()

            if svtype == "ext":
                info.mapcrc = int(slots.pop(0))
                info.mapsize = int(slots.pop(0))

            info.game_type = slots.pop(0).decode()
            info.password = int.from_bytes(slots.pop(0), sys.byteorder) == 1
            info.player_count = int(slots.pop(0))
            info.max_player_count = int(slots.pop(0))
            info.client_count = int(slots.pop(0))
            info.max_client_count = int(slots.pop(0))

        if svtype == "ext":
            slots.pop(0)  # reserved

        for i in range(info.player_count):
            info.players.append(Player(slots.pop(0).decode(),
                                       slots.pop(0).decode(),
                                       int(slots.pop(0)),
                                       int(slots.pop(0)),
                                       int(slots.pop(0)) == 0)
                                )
            if svtype == "ext":
                slots.pop(0)  # reserved
        return info



from pygments.lexers._openedge_builtins import OPENEDGEKEYWORDS
from setuptools import setup

setup(
    name='twstatus.py',
    version='0.1.1',
    packages=["twstatus"],
    url='https://github.com/Ryozuki/twstatus.py',
    license='MIT',
    author='Edgar',
    author_email='',
    description='Get information about teeworlds/ddnet servers.',
    zip_safe=False,
    keywords=['teeworlds', 'servers', 'info', 'asyncio'],
    python_requires='>=3.6',
    classifiers=[
        # How mature is this project? Common values are
        #   1 - Planning
        #   2 - Pre-Alpha
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 4 - Beta',

        # Indicate who your project is intended for
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Topic :: Utilities',
        'Topic :: Software Development :: Libraries',
        'Topic :: Software Development :: Libraries :: Python Modules',

        # Pick your license as you wish (should match "license" above)
        'License :: OSI Approved :: MIT License',

        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        'Programming Language :: Python :: 3.6',
    ]
)
